# perform gstreamer imports (python3 style)
import gi
gi.require_version('Gst','1.0')

from gi.repository import GObject, Gst

from threading import Thread
import time

# import from this module 
from . basicplayer import BasicPlayer

class Monitor:
    def __init__(self,player):
        self.player = player
        #Thread.__init__(self)
        self.daemon = True      # as in: don't wait on this thread to quit
        self._running = False

    def stop(self):
        if self._running:
            self._running = False
            #self.join(0.5)
    
    def start(self):
        self._running = True
        self.run()

    def run(self):
        if self.player.player_state == "PLAYING":
            position = self.player.position()

            # handle finishing of limited playtime
            if self.player.playtime > 0:
                if (position - self.player.startpos) > self.player.playtime:
                    #self._player.pause(notify=False)
                    print("Finished limited play")
                    self.player.finished()

        if self._running and time is not None:
            GObject.timeout_add(100, self.run)

class MonitoredPlayer(BasicPlayer):
    def __init__(self):
        BasicPlayer.__init__(self)
        # setup monitor thread
        self.monitor = Monitor(self)
        self.monitor.start()
        self.playtime = -1
        self.looping=False
        
    def __del__(self):
        if hasattr(self,'monitor'):
            self.monitor.stop()
        
    def playfor(self,duration,loop=False):
        loop = False
        if duration > 0:    # not much point in wasting system resources otherwise
            self.startpos = self.position()
            self.playtime = duration
            BasicPlayer.play(self)
        elif duration < 0: # same as normal play
            self.play()

    def loop(self):
        self.looping = True
        if self.player_state != "PAUSED":
            self.startpos = 0
            self.playtime = -1
        BasicPlayer.play(self)


    def play(self):
        self.looping = False
        if self.player_state != "PAUSED":
            self.startpos = 0
            self.playtime = -1
        BasicPlayer.play(self)

    def stop(self):
        self.playtime = -1
        BasicPlayer.stop(self)
        #self.monitor.stop()


    def finished(self):
        if self.looping:
            print("Finished - looping")
            self.seek(0)
            if self.player_state in ["READY","PAUSED",]:
                self.player_state = "PLAYING"
                self.pipeline.set_state(Gst.State.PLAYING)
        else:
            BasicPlayer.finished(self)  
          

GObject.type_register(MonitoredPlayer)
    